import React from 'react';
import {
  Container,
  Row,
  Col,
  ListGroup,
  ListGroupItem,
  Badge,
  Tooltip
} from 'reactstrap';
import Api from './Api';
import {
  FaMapMarkerAlt
} from 'react-icons/fa';
import {
  NotificationContainer,
  NotificationManager
} from 'react-notifications';
import {
  Link
} from 'react-router-dom';
import strToSlug from '../StrToSlug'

import '../css/Map.css'

const MapDimensions = {
  width: 734,
  height: 530,
};

export default class Map extends React.Component {
  constructor(props) {
    super(props);

    this.state = {
      types: [],
      vegetables: [],
      selectedType: {},
      selectedVegetable: {},
      Map: {
        width: '20px',
        height: '20px',
      },
      tooltipOpen: false
    }

    this.getVegetablesTypes = this.getVegetablesTypes.bind(this);
    this.selectType = this.selectType.bind(this);
    this.selectVegetable = this.selectVegetable.bind(this);
    this.setMap = this.setMap.bind(this);
    this.toggle = this.toggle.bind(this);

    this.getVegetablesTypes();
  }

  componentWillReceiveProps(nextProps) {
    setTimeout(() => {
      this.setMap();
    }, 200);
  }

  componentDidMount() {
    window.addEventListener('resize', this.setMap);
    this.setMap();
  }

  componentWillUnmount() {
    window.removeEventListener('resize', this.setMap);
  }

  setMap() {
    const width = document.getElementById('MapContainer').clientWidth;
    let MapWidth = MapDimensions.width;
    let MapHeight = MapDimensions.height;

    console.log('Update map size')

    if (width < MapDimensions.width) {
      MapWidth = width;
      MapHeight = Math.round(MapDimensions.height * MapWidth / MapDimensions.width);
    }

    this.setState({
      Map: {
        width: MapWidth,
        height: MapHeight,
      },
    });
  }

  getVegetablesTypes() {
    Api.get('types')
      .then(res => {
        this.setState({
          types: res.data.rows
        });
      })
      .catch(() => {
        NotificationManager.error('Erreur lors de la récupération des catégories');
      })
  }

  getVegetable(id) {
    Api.get(`types/${id}`)
      .then(res => {
        console.log(res.data)
        this.setState({
          vegetables: res.data
        });
      })
      .catch(() => {
        NotificationManager.error('Erreur lors de la récupération des végétaux');
      })
  }

  selectType(type) {
    this.setState({
      selectedType: type
    });
  }

  selectVegetable(vegetable) {
    this.setState({
      selectedVegetable: vegetable
    })
  }

  toggle(id) {
    this.setState(prevState => ({
      tooltipOpen: prevState.tooltipOpen === id ? false : id
    }));
  }

  render() {
    return (
      <Container>
        <NotificationContainer />
        <Row>
          <Col xs="12" sm="8">
            <Row className="with-margin">
              <Col className="with-border with-background">
                <div id="MapContainer">
                  <div className="map" style={{ width: this.state.Map.width, height: this.state.Map.height }}>
                    {
                      this.state.selectedType &&
                      this.state.selectedType.Vegetables &&
                      this.state.selectedType.Vegetables.map((vegetable, key) =>
                        (
                          <Link to={`/vegetaux/${this.state.selectedType.id}-${strToSlug(this.state.selectedType.name)}/${vegetable.id}-${strToSlug(vegetable.name)}`}
                            key={key}
                            className={`mapMarker ${this.state.selectedVegetable && this.state.selectedVegetable.id === vegetable.id ? 'selected' : ''}`}
                            style={{ left: `calc(${vegetable.lat}% - 16px)`, top: `calc(${vegetable.lng}% - 16px)` }}
                            onMouseOver={(e) => this.selectVegetable(vegetable)}
                          >
                            <Tooltip placement='top' isOpen={this.state.tooltipOpen === vegetable.id} target={'Tooltip-' + vegetable.id} toggle={() => this.toggle(vegetable.id)}>
                              {vegetable.name}
                            </Tooltip>
                            <FaMapMarkerAlt id={'Tooltip-' + vegetable.id} />
                          </Link>
                        )
                      )
                    }
                  </div>
                </div>
              </Col>
            </Row>
          </Col>
          <Col xs="12" sm="4">
            <Row>
              <Col xs="6" sm="12">
                <div className="with-margin">
                  <div className=" with-border with-background">
                    <ListGroup className='vegetables--types--group'>
                      {
                        this.state.types.map((type, key) =>
                          (
                            <ListGroupItem
                              key={key}
                              className={this.state.selectedType.id === type.id ? "selected" : "null"} onClick={(e) => this.selectType(type)}>
                              {type.name}
                              <Badge pill>{type.Vegetables.length}</Badge>
                            </ListGroupItem>
                          )
                        )
                      }
                    </ListGroup>
                  </div>
                </div>
              </Col>
              <Col xs="6" sm="12">
                <div className="with-margin">
                  <div className=" with-border with-background">
                    <ListGroup className='vegetables--types--group'>
                      {
                        this.state.selectedType &&
                        this.state.selectedType.Vegetables &&
                        this.state.selectedType.Vegetables.map((vegetable, key) =>
                          (
                            <ListGroupItem key={key} className={this.state.selectedVegetable.id === vegetable.id ? "selected" : "null"} onMouseOver={(e) => this.selectVegetable(vegetable)}>
                              {vegetable.name}
                            </ListGroupItem>
                          )
                        )
                      }
                    </ListGroup>
                  </div>
                </div>
              </Col>
            </Row>
          </Col>
        </Row>
      </Container>
    );
  }
}